class CousteauGenericError(Exception):
    """Custom Exception class for cousteau general erorrs."""
    pass


class APIResponseError(Exception):
    """Custom Exception class for errors in ATLAS API responses."""
    pass
